@@ -3,16 +3,18 @@ ANDES command-line interface and argument parsers.
 """
 
 import argparse
-import sys
-import os
-import logging
-import platform
 import importlib
-
+import logging
+import os
+import platform
+import sys
 from time import strftime
+
+from ._version import get_versions
+
 from andes.main import config_logger, find_log_path
-from andes.utils.paths import get_log_dir
 from andes.routines import routine_cli
+from andes.utils.paths import get_log_dir
 
 logger = logging.getLogger(__name__)
 
@@ -34,6 +36,9 @@ def create_parser():
              'or 40-ERROR.',
         type=int, default=20, choices=(1, 10, 20, 30, 40))
 
+    parser.add_argument(
+        '--version', help='show version info and exit', action='store_true')
+
     sub_parsers = parser.add_subparsers(dest='command', help='[run] run simulation routine; '
                                                              '[plot] plot results; '
                                                              '[doc] quick documentation; '
@@ -139,6 +144,8 @@ def create_parser():
     prep_mode.add_argument('-f', '--full', action='store_true', help='full codegen')
     prep_mode.add_argument('-i', '--incremental', action='store_true',
                            help='rapid incrementally generate for updated models')
+    prep.add_argument('-c', '--compile', help='compile the code with numba after codegen',
+                      action='store_true', dest='precompile')
     prep.add_argument('--pycode-path', help='Save path for generated pycode')
     prep.add_argument('-m', '--models', nargs='*', help='model names to be individually prepared',
                       )
@@ -178,10 +185,18 @@ def preamble():
 
 
 def main():
-    """Main command-line interface"""
+    """
+    Main command-line interface
+    """
+
     parser = create_parser()
     args = parser.parse_args()
 
+    if args.version is True:
+        print(get_versions()['version'])
+        return
+
+    # Set up logging
     config_logger(stream=True,
                   stream_level=args.verbose,
                   file=True,
@@ -198,6 +213,7 @@ def main():
     else:
         preamble()
 
+    # Run the command
     if args.command is None:
         parser.parse_args(sys.argv.append('--help'))
 
