@@ -11,29 +11,26 @@ Base class for building ANDES models.
 #  File name: model.py
 
 import logging
-
 from collections import OrderedDict
-from typing import Iterable, Sized
+from typing import Iterable, Sized, Union, Callable
 
 import scipy as sp
 
 from andes.core.block import Block
-from andes.core.common import ModelFlags, JacTriplet, Config
+from andes.core.common import Config, JacTriplet, ModelFlags
 from andes.core.discrete import Discrete
 from andes.core.documenter import Documenter
-from andes.core.param import BaseParam, IdxParam, DataParam, NumParam, ExtParam, TimerParam
-from andes.core.service import BaseService, ConstService, BackRef, VarService, PostInitService
-from andes.core.service import ExtService, NumRepeat, NumReduce, RandomService, DeviceFinder
-from andes.core.service import NumSelect, FlagValue, ParamCalc, InitChecker, Replace, ApplyFunc
-from andes.core.service import SwBlock
+from andes.core.param import (BaseParam, DataParam, ExtParam, IdxParam,
+                              NumParam, TimerParam, )
+from andes.core.service import (ApplyFunc, BackRef, BaseService, ConstService,
+                                DeviceFinder, ExtService, FlagValue,
+                                InitChecker, NumReduce, NumRepeat, NumSelect,
+                                ParamCalc, PostInitService, RandomService,
+                                Replace, SwBlock, VarService, )
 from andes.core.symprocessor import SymProcessor
-from andes.core.var import BaseVar, Algeb, State, ExtAlgeb, ExtState
-
-from andes.shared import jac_names, jac_types, jac_full_names
-from andes.shared import np, pd
-
+from andes.core.var import Algeb, BaseVar, ExtAlgeb, ExtState, State
+from andes.shared import jac_full_names, jac_names, jac_types, np, pd, numba
 from andes.utils.func import list_flatten
-from andes.utils.numba import to_jit
 
 logger = logging.getLogger(__name__)
 
@@ -58,6 +55,9 @@ class ModelCache:
 
         return self.__getattribute__(item)
 
+    def __getstate__(self):
+        return self.__dict__
+
     def add_callback(self, name: str, callback):
         """
         Add a cache attribute and a callback function for updating the attribute.
@@ -79,6 +79,7 @@ class ModelCache:
         ----------
         name : str, list, optional
             name or list of cached to refresh, by default None for refreshing all
+
         """
         if name is None:
             for name in self._callbacks.keys():
@@ -119,8 +120,8 @@ class ModelData:
     Models should inherit this class to define the parameters from input files.
 
     Inherit this class to create the specific class for holding input parameters for a new model.
-    The recommended name for the derived class is the model name with ``Data``. For example, data for `GENCLS`
-    should be named `GENCLSData`.
+    The recommended name for the derived class is the model name with ``Data``. For example, data for `GENROU`
+    should be named `GENROUData`.
 
     Parameters should be defined in the ``__init__`` function of the derived class.
 
@@ -428,8 +429,8 @@ class ModelCall:
         self.ii = OrderedDict()
         self.ij = OrderedDict()
         self.s_args = OrderedDict()
-        self.ia_args = OrderedDict()   # assignment initialization
-        self.ii_args = OrderedDict()   # iterative initialization
+        self.ia_args = OrderedDict()  # assignment initialization
+        self.ii_args = OrderedDict()  # iterative initialization
         self.ij_args = OrderedDict()
 
         self.ijac = OrderedDict()
@@ -622,7 +623,11 @@ class Model:
 
         self.tex_names = OrderedDict((('dae_t', 't_{dae}'),
                                       ('sys_f', 'f_{sys}'),
-                                      ('sys_mva', 'S_{b,sys}')
+                                      ('sys_mva', 'S_{b,sys}'),
+                                      ('__zeros', '0'),
+                                      ('__ones', '1'),
+                                      ('__falses', '0'),
+                                      ('__trues', '1'),
                                       ))
 
         # Model behavior flags
@@ -657,13 +662,13 @@ class Model:
         self.cache.add_callback('e_adders', self._e_adders)
         self.cache.add_callback('e_setters', self._e_setters)
 
-        self._input = OrderedDict()    # cached dictionary of inputs
-        self._input_z = OrderedDict()  # discrete flags in an OrderedDict
-        self._rhs_f = OrderedDict()    # RHS of external f
-        self._rhs_g = OrderedDict()    # RHS of external g
+        self._input = OrderedDict()  # cached dictionary of inputs
+        self._input_z = OrderedDict()  # discrete flags, storage only.
+        self._rhs_f = OrderedDict()  # RHS of external f
+        self._rhs_g = OrderedDict()  # RHS of external g
 
         self.f_args = []
-        self.g_args = []   # argument value lists
+        self.g_args = []  # argument value lists
         self.j_args = dict()
         self.s_args = OrderedDict()
         self.ia_args = OrderedDict()
@@ -792,6 +797,17 @@ class Model:
 
         return self.uid[idx]
 
+    def set_backref(self, name, from_idx, to_idx):
+        """
+        Helper function for setting idx-es to ``BackRef``.
+        """
+
+        if name not in self.services_ref:
+            return
+
+        uid = self.idx2uid(to_idx)
+        self.services_ref[name].v[uid].append(from_idx)
+
     def get(self, src: str, idx, attr: str = 'v', allow_none=False, default=0.0):
         """
         Get the value of an attribute of a model property.
@@ -946,14 +962,49 @@ class Model:
         for instance in self.cache.all_vars.values():
             self._input[instance.name] = instance.v
 
-        # append config variables
+        # append config variables as arrays
         for key, val in self.config.as_dict(refresh=True).items():
             self._input[key] = np.array(val)
 
-        # update`dae_t` and `sys_f`
+        # zeros and ones
+        self._input['__zeros'] = np.zeros(self.n)
+        self._input['__ones'] = np.ones(self.n)
+        self._input['__falses'] = np.full(self.n, False)
+        self._input['__trues'] = np.full(self.n, True)
+
+        # --- below are numpy scalars ---
+        # update`dae_t` and `sys_f`, and `sys_mva`
+        self._input['sys_f'] = np.array(self.system.config.freq, dtype=float)
+        self._input['sys_mva'] = np.array(self.system.config.mva, dtype=float)
         self._input['dae_t'] = self.system.dae.t
-        self._input['sys_f'] = self.system.config.freq
-        self._input['sys_mva'] = self.system.config.mva
+
+    def mock_refresh_inputs(self):
+        """
+        Use mock data to fill the inputs.
+
+        This function is used to generate input data of the desired type
+        to trigget JIT compilation.
+        """
+
+        self.get_inputs()
+        mock_arr = np.array([1.])
+
+        for key in self._input.keys():
+            try:
+                key_ndim = self._input[key].ndim
+            except AttributeError:
+                logger.error(key)
+                logger.error(self.class_name)
+
+            key_type = self._input[key].dtype
+
+            if key_ndim == 0:
+                self._input[key] = mock_arr.reshape(()).astype(key_type)
+            elif key_ndim == 1:
+                self._input[key] = mock_arr.astype(key_type)
+
+            else:
+                raise NotImplementedError("Unkonwn input data dimension %s" % key_ndim)
 
     def refresh_inputs_arg(self):
         """
@@ -1263,16 +1314,10 @@ class Model:
             for idx, fun in enumerate(self.calls.vjac[jname]):
                 try:
                     self.triplets.vjac[jname][idx][:] = ret[idx]
-                except ValueError as e:
+                except (ValueError, IndexError, FloatingPointError) as e:
                     row_name, col_name = self._jac_eq_var_name(jname, idx)
-                    logger.error('%s shape error: j_idx=%s, d%s / d%s',
-                                 jname, idx, row_name, col_name)
-
-                    raise e
-                except FloatingPointError as e:
-                    row_name, col_name = self._jac_eq_var_name(jname, idx)
-                    logger.error('%s eval error: j_idx=%s, d%s / d%s',
-                                 jname, idx, row_name, col_name)
+                    logger.error('%s: error calculating or storing Jacobian <%s>: j_idx=%s, d%s / d%s',
+                                 self.class_name, jname, idx, row_name, col_name)
 
                     raise e
 
@@ -1605,7 +1650,7 @@ class Model:
             for item in self.services_icheck.values():
                 item.check()
 
-    def numba_jitify(self, parallel=False, cache=False, nopython=False):
+    def numba_jitify(self, parallel=False, cache=True, nopython=False):
         """
         Optionally convert `self.calls.f` and `self.calls.g` to
         JIT compiled functions.
@@ -1624,23 +1669,49 @@ class Model:
         if self.flags.jited is True:
             return
 
-        self.calls.f = to_jit(self.calls.f,
-                              parallel=parallel,
-                              cache=cache,
-                              nopython=nopython,
-                              )
-        self.calls.g = to_jit(self.calls.g,
-                              parallel=parallel,
-                              cache=cache,
-                              nopython=nopython,
-                              )
+        kwargs = {'parallel': parallel,
+                  'cache': cache,
+                  'nopython': nopython,
+                  }
+
+        self.calls.f = to_jit(self.calls.f, **kwargs)
+        self.calls.g = to_jit(self.calls.g, **kwargs)
 
         for jname in self.calls.j:
-            self.calls.j[jname] = to_jit(self.calls.j[jname],
-                                         parallel=parallel, cache=cache)
+            self.calls.j[jname] = to_jit(self.calls.j[jname], **kwargs)
+
+        for name, instance in self.services_var.items():
+            if instance.v_str is not None:
+                self.calls.s[name] = to_jit(self.calls.s[name], **kwargs)
 
         self.flags.jited = True
 
+    def precompile(self):
+        """
+        Trigger numba compilation for this model.
+
+        This function requires the system to be setup, i.e.,
+        memory allocated for storage.
+        """
+
+        self.get_inputs()
+        if self.n == 0:
+            self.mock_refresh_inputs()
+        self.refresh_inputs_arg()
+
+        if callable(self.calls.f):
+            self.calls.f(*self.f_args)
+
+        if callable(self.calls.g):
+            self.calls.g(*self.g_args)
+
+        for jname, jfunc in self.calls.j.items():
+            jfunc(*self.j_args[jname])
+
+        for name, instance in self.services_var.items():
+            if instance.v_str is not None:
+                self.calls.s[name](*self.s_args[name])
+
     def __repr__(self):
         dev_text = 'device' if self.n == 1 else 'devices'
 
@@ -1673,12 +1744,20 @@ class Model:
             kwargs = self.get_inputs(refresh=True)
 
             for idx, name in enumerate(self.calls.init_seq):
-                # single variable
+                # single variable - do assignment
                 if isinstance(name, str):
                     instance = self.__dict__[name]
                     _eval_discrete(instance)
                     if instance.v_str is not None:
-                        instance.v[:] = self.calls.ia[name](*self.ia_args[name])
+                        if not instance.v_str_add:
+                            # assignment is for most variable initialization
+                            instance.v[:] = self.calls.ia[name](*self.ia_args[name])
+                        else:
+                            # in-place add initial values.
+                            # Voltage compensators can set part of the `v` of exciters.
+                            # Exciters will then set the bus voltage part.
+                            instance.v[:] += self.calls.ia[name](*self.ia_args[name])
+
                     # single variable iterative solution
                     if name in self.calls.ii:
                         self.solve_iter(name, kwargs)
@@ -1760,17 +1839,50 @@ class Model:
         else:
             logger.warning(f"{self.class_name}: iterative initialization failed for {self.idx.v[pos]}.")
 
+    def externalize(self):
+        """
+        Externalize internal data as a snapshot.
+        """
+        pass
+
+    def internalize(self):
+        """
+        Internalize snapshot data.
+        """
+        pass
+
 
 def _eval_discrete(instance):
-
     # for variables associated with limiters, limiters need to be evaluated
     # before variable initialization.
     # However, if any limit is hit, initialization is likely to fail.
 
     if instance.discrete is not None:
         if not isinstance(instance.discrete, (list, tuple, set)):
-            dlist = (instance.discrete, )
+            dlist = (instance.discrete,)
         else:
             dlist = instance.discrete
         for d in dlist:
             d.check_var()
+
+
+def to_jit(func: Union[Callable, None],
+           parallel: bool = False,
+           cache: bool = False,
+           nopython: bool = False,
+           ):
+    """
+    Helper function for converting a function to a numba jit-compiled function.
+
+    Note that this function will be compiled just-in-time when first called,
+    based on the argument types.
+    """
+
+    if func is not None:
+        return numba.jit(func,
+                         parallel=parallel,
+                         cache=cache,
+                         nopython=nopython,
+                         )
+
+    return func
