@@ -1,8 +1,9 @@
 import logging
-import numpy as np
 from collections import OrderedDict
-from andes.core.service import BackRef
 
+import numpy as np
+
+from andes.core.service import BackRef
 from andes.utils.func import list_flatten
 
 logger = logging.getLogger(__name__)
@@ -336,6 +337,17 @@ class GroupBase:
 
         return ret
 
+    def set_backref(self, name, from_idx, to_idx):
+        """
+        Set idxes to ``BackRef``, and set them to models.
+        """
+
+        uid = self.idx2uid(to_idx)
+        self.services_ref[name].v[uid].append(from_idx)
+
+        model = self.idx2model(to_idx)
+        model.set_backref(name, from_idx, to_idx)
+
     def get_next_idx(self, idx=None, model_name=None):
         """
         Get a no-conflict idx for a new device.
@@ -647,9 +659,21 @@ class Exciter(GroupBase):
         self.common_params.extend(('syn',))
         self.common_vars.extend(('vout', 'vi',))
 
+        self.VoltComp = BackRef()
         self.PSS = BackRef()
 
 
+class VoltComp(GroupBase):
+    """
+    Voltage compensator group for synchronous generators.
+    """
+
+    def __init__(self):
+        super().__init__()
+        self.common_params.extend(('rc', 'xc',))
+        self.common_vars.extend(('vcomp',))
+
+
 class PSS(GroupBase):
     """Power system stabilizer group."""
 
