@@ -2,11 +2,13 @@ from collections import OrderedDict
 
 import numpy as np
 
-from andes.core import ModelData, IdxParam, NumParam, Model, Switcher, ExtParam, ExtAlgeb, ExtService, \
-    ConstService, \
-    Limiter, Lag, Algeb, Piecewise
-from andes.core.block import PITrackAWFreeze, GainLimiter, LagFreeze, DeadBand1, LagRate, LagAWFreeze
-from andes.core.service import Replace, DataSelect, VarService, ExtendedEvent, VarHold, ApplyFunc
+from andes.core import (Algeb, ConstService, ExtAlgeb, ExtParam, ExtService,
+                        IdxParam, Lag, Limiter, Model, ModelData, NumParam,
+                        Piecewise, Switcher,)
+from andes.core.block import (DeadBand1, GainLimiter, LagAWFreeze, LagFreeze,
+                              LagRate, PITrackAWFreeze,)
+from andes.core.service import (ApplyFunc, DataSelect, ExtendedEvent, Replace,
+                                VarHold, VarService,)
 from andes.core.var import AliasState
 
 
@@ -588,11 +590,13 @@ class REECA1Model(Model):
         Ipmax2sq = '(Imax**2 - IqHL_y**2)'
 
         # `Ipmax20`-squared (non-negative)
-        self.Ipmax2sq0 = ConstService(v_str=f'Piecewise((0.0, Le({Ipmax2sq0}, 0.0)), ({Ipmax2sq0}, True))',
+        self.Ipmax2sq0 = ConstService(v_str=f'Piecewise((0, Le({Ipmax2sq0}, 0.0)), ({Ipmax2sq0}, True), \
+                                              evaluate=False)',
                                       tex_name='I_{pmax20,nn}^2',
                                       )
 
-        self.Ipmax2sq = VarService(v_str=f'Piecewise((0.0, Le({Ipmax2sq}, 0.0)), ({Ipmax2sq}, True))',
+        self.Ipmax2sq = VarService(v_str=f'Piecewise((0, Le({Ipmax2sq}, 0.0)), ({Ipmax2sq}, True), \
+                                           evaluate=False)',
                                    tex_name='I_{pmax2}^2',
                                    )
 
@@ -613,11 +617,13 @@ class REECA1Model(Model):
 
         Iqmax2sq0 = '(Imax**2 - Ipcmd0**2)'  # initialization equation by using `Ipcmd0`
 
-        self.Iqmax2sq0 = ConstService(v_str=f'Piecewise((0.0, Le({Iqmax2sq0}, 0.0)), ({Iqmax2sq0}, True))',
+        self.Iqmax2sq0 = ConstService(v_str=f'Piecewise((0, Le({Iqmax2sq0}, 0.0)), ({Iqmax2sq0}, True), \
+                                              evaluate=False)',
                                       tex_name='I_{qmax,nn}^2',
                                       )
 
-        self.Iqmax2sq = VarService(v_str=f'Piecewise((0.0, Le({Iqmax2sq}, 0.0)), ({Iqmax2sq}, True))',
+        self.Iqmax2sq = VarService(v_str=f'Piecewise((0, Le({Iqmax2sq}, 0.0)), ({Iqmax2sq}, True), \
+                                           evaluate=False)',
                                    tex_name='I_{qmax2}^2')
 
         self.Iqmax = Algeb(v_str=f'(SWPQ_s0*{Iqmax1} + SWPQ_s1*sqrt(Iqmax2sq0))',
