@@ -13,34 +13,32 @@ System class for power system data and methods
 
 import configparser
 import importlib
+import inspect
 import logging
 import os
 import sys
-import inspect
+import time
+from collections import OrderedDict
+from typing import Dict, Optional, Tuple, Union
+
 import dill
 
-from collections import OrderedDict
-from typing import Dict, Tuple, Union, Optional
-
 import andes.io
+from andes.core import AntiWindup, Config, Model
+from andes.io.streaming import Streaming
 from andes.models import file_classes
 from andes.models.group import GroupBase
-from andes.variables import FileMan, DAE
 from andes.routines import all_routines
-from andes.utils.tab import Tab
+from andes.shared import (Pool, Process, dilled_vars, jac_names, matrix, np,
+                          sparse, spmatrix,)
 from andes.utils.misc import elapsed
-from andes.utils.paths import confirm_overwrite
-from andes.utils.paths import get_config_path, get_dot_andes_path
-from andes.utils.paths import get_pycode_path, get_pkl_path
-from andes.core import Config, Model, AntiWindup
-from andes.io.streaming import Streaming
-
-from andes.shared import np, jac_names, dilled_vars, IP_ADD
-from andes.shared import matrix, spmatrix, sparse, Pool
+from andes.utils.paths import (andes_root, confirm_overwrite, get_config_path,
+                               get_pkl_path, get_pycode_path,)
+from andes.utils.tab import Tab
+from andes.variables import DAE, FileMan
 
 logger = logging.getLogger(__name__)
 dill.settings['recurse'] = True
-pycode = None
 
 
 class ExistingModels:
@@ -62,6 +60,12 @@ class System:
     These attributes include `models`, `groups`, `routines` and `calls` for loaded models, groups,
     analysis routines, and generated numerical function calls, respectively.
 
+    Parameters
+    ----------
+    no_undill : bool, optional
+        True to disable the call to ``System.undill()`` at the end of object creation.
+        False by default.
+
     Notes
     -----
     System stores model and routine instances as attributes.
@@ -93,6 +97,7 @@ class System:
                  config_path: Optional[str] = None,
                  default_config: Optional[bool] = False,
                  options: Optional[Dict] = None,
+                 no_undill: Optional[bool] = False,
                  **kwargs
                  ):
         self.name = name
@@ -135,7 +140,6 @@ class System:
                                      ('dime_address', 'ipc:///tmp/dime2'),
                                      ('numba', 0),
                                      ('numba_parallel', 0),
-                                     ('numba_cache', 1),
                                      ('numba_nopython', 0),
                                      ('yapf_pycode', 0),
                                      ('np_divide', 'warn'),
@@ -152,7 +156,6 @@ class System:
                               warn_abnormal='warn initialization out of normal values',
                               numba='use numba for JIT compilation',
                               numba_parallel='enable parallel for numba.jit',
-                              numba_cache='enable machine code caching for numba.jit',
                               numba_nopython='nopython mode for numba',
                               yapf_pycode='format generated code with yapf',
                               np_divide='treatment for division by zero',
@@ -168,7 +171,6 @@ class System:
                               warn_abnormal=(0, 1),
                               numba=(0, 1),
                               numba_parallel=(0, 1),
-                              numba_cache=(0, 1),
                               numba_nopython=(0, 1),
                               yapf_pycode=(0, 1),
                               np_divide={'ignore', 'warn', 'raise', 'call', 'print', 'log'},
@@ -197,6 +199,9 @@ class System:
         # internal flags
         self.is_setup = False        # if system has been setup
 
+        if not no_undill:
+            self.undill()
+
     def _set_numpy(self):
         """
         Configure NumPy based on Config.
@@ -214,6 +219,7 @@ class System:
         """
         Reload a new case in the same System object.
         """
+
         self.options.update(kwargs)
         self.files.set(case=case, **kwargs)
         # TODO: clear all flags and empty data
@@ -269,7 +275,6 @@ class System:
         Generated lambda functions will be serialized to file, but pretty prints (SymPy objects) can only exist in
         the System instance on which prepare is called.
         """
-        # info
         if incremental is True:
             mode_text = 'rapid incremental mode'
         elif quick is True:
@@ -295,7 +300,7 @@ class System:
         elif not incremental and models is None:
             models = self.models
         else:
-            models = self._to_orddct(models)
+            models = self._get_models(models)
 
         total = len(models)
         width = len(str(total))
@@ -368,8 +373,7 @@ class System:
         logger.info('Saved generated pycode to "%s"', pycode_path)
 
         # RELOAD REQUIRED as the generated Jacobian arguments may be in a different order
-        if pycode:
-            self._call_from_pycode()
+        self._call_from_pycode()
 
     def _find_stale_models(self):
         """
@@ -428,7 +432,7 @@ class System:
 
         # assign address at the end before adding devices and processing parameters
         self.set_address(self.exist.pflow)
-        self.set_dae_names(self.exist.pflow)
+        self.set_dae_names(self.exist.pflow)        # needs perf. optimization
         self.store_sparse_pattern(self.exist.pflow)
         self.store_adder_setter(self.exist.pflow)
 
@@ -484,6 +488,7 @@ class System:
 
         if self.is_setup:
             raise NotImplementedError("Adding devices are not allowed after setup.")
+
         group_name = self.__dict__[model].group
         group = self.groups[group_name]
 
@@ -613,7 +618,11 @@ class System:
 
     def set_var_arrays(self, models, inplace=True, alloc=True):
         """
-        Set arrays (`v` and `e`) in internal variables.
+        Set arrays (`v` and `e`) for internal variables to access
+        dae arrays in place.
+
+        This function needs to be called after de-serializing a System object,
+        where the internal variables are incorrectly assigned new memory.
 
         Parameters
         ----------
@@ -639,18 +648,77 @@ class System:
         """
         Helper function to compile all functions with Numba before init.
         """
-        if self.config.numba:
-            use_parallel = bool(self.config.numba_parallel)
-            use_cache = bool(self.config.numba_cache)
-            nopython = bool(self.config.numba_nopython)
 
-            logger.info("Numba compilation initiated, parallel=%s, cache=%s.",
-                        use_parallel, use_cache)
-            for mdl in models.values():
-                mdl.numba_jitify(parallel=use_parallel,
-                                 cache=use_cache,
-                                 nopython=nopython,
-                                 )
+        if not self.config.numba:
+            return
+
+        use_parallel = bool(self.config.numba_parallel)
+        nopython = bool(self.config.numba_nopython)
+
+        logger.info("Numba compilation initiated with caching. Parallel=%s.",
+                    use_parallel)
+
+        for mdl in models.values():
+            mdl.numba_jitify(parallel=use_parallel,
+                             nopython=nopython,
+                             )
+
+    def precompile(self,
+                   models: Union[OrderedDict, None] = None,
+                   nomp: bool = False,
+                   ncpu: int = os.cpu_count()):
+        """
+        Trigger precompilation for the given models.
+
+        Arguments are the same as ``prepare``.
+        """
+
+        t0, _ = elapsed()
+
+        if models is None:
+            models = self.models
+        else:
+            models = self._get_models(models)
+
+        # turn on numba for precompilation
+        self.config.numba = 1
+
+        self.setup()
+        self._init_numba(models)
+
+        def _precompile_model(model: Model):
+            model.precompile()
+
+        logger.info("Compilation in progress. This might take a minute...")
+
+        if nomp is True:
+            for name, mdl in models.items():
+                _precompile_model(mdl)
+                logger.debug("Model <%s> compiled.", name)
+
+        # multi-processed implementation. `Pool.map` runs very slow somehow.
+        else:
+            jobs = []
+            for idx, (name, mdl) in enumerate(models.items()):
+                job = Process(
+                    name='Process {0:d}'.format(idx),
+                    target=_precompile_model,
+                    args=(mdl,),
+                    )
+                jobs.append(job)
+                job.start()
+
+                if (idx % ncpu == ncpu - 1) or (idx == len(models) - 1):
+                    time.sleep(0.02)
+                    for job in jobs:
+                        job.join()
+                    jobs = []
+
+        _, s = elapsed(t0)
+        logger.info('Numba compiled %d model%s in %s.',
+                    len(models),
+                    '' if len(models) == 1 else 's',
+                    s)
 
     def init(self, models: OrderedDict, routine: str):
         """
@@ -662,13 +730,11 @@ class System:
         - Call the model `init()` method, which initializes internal variables.
         - Copy variables to DAE and then back to the model.
         """
-        try:
-            self._init_numba(models)
-        except ImportError:
-            logger.warning("Numba not found. JIT compilation is skipped.")
+
+        self._init_numba(models)
 
         for mdl in models.values():
-            # link externals first
+            # link externals services first
             for instance in mdl.services_ext.values():
                 ext_name = instance.model
                 try:
@@ -708,6 +774,12 @@ class System:
             if not mdl.n:
                 continue
 
+            # Fixes an issue if the cache was manually built but stale
+            # after assigning addresses for simulation
+            # Assigning memory will affect the cache of `v_adders` and `e_adders`.
+
+            mdl.cache.refresh()
+
             # ``getters` that retrieve variable values from DAE
             for var in mdl.cache.v_getters.values():
                 self._getters[var.v_code].append(var)
@@ -941,7 +1013,7 @@ class System:
             for mdl in models.values():
                 for rows, cols, vals in mdl.triplets.zip_ijv(j_name):
                     try:
-                        if self.config.ipadd and IP_ADD:
+                        if self.config.ipadd:
                             self.dae.__dict__[j_name].ipadd(vals, rows, cols)
                         else:
                             self.dae.__dict__[j_name] += spmatrix(vals, rows, cols, j_size, 'd')
@@ -968,7 +1040,7 @@ class System:
         aidx = self.Bus.islanded_a
         vidx = self.Bus.islanded_v
 
-        if self.config.ipadd and IP_ADD:
+        if self.config.ipadd:
             self.dae.gy.ipset(self.config.diag_eps, aidx, aidx)
             self.dae.gy.ipset(self.config.diag_eps, vidx, vidx)
         else:
@@ -1340,7 +1412,8 @@ class System:
 
         """
         np_ver = np.__version__.split('.')
-        np_ver = tuple([int(i) for i in np_ver])
+        # Read only first two elements. Last one may contain 'rcxx'
+        np_ver = tuple([int(i) for i in np_ver[:2]])
         if np_ver < (1, 20):
             logger.debug("Dumping calls to calls.pkl with dill")
             dill.settings['recurse'] = True
@@ -1410,38 +1483,51 @@ class System:
         Helper function for loading ModelCall from pickle file.
         """
 
-        self.calls = self._load_pkl()
         loaded = False
+        any_calls = self._load_pkl()
 
-        if self.calls is not None:
-            loaded = True
+        if any_calls is not None:
+            self.calls = any_calls
 
             for name, model_call in self.calls.items():
                 if name in self.__dict__:
                     self.__dict__[name].calls = model_call
 
+            loaded = True
+
         return loaded
 
     def _call_from_pycode(self):
         """
         Helper function to import generated pycode.
+
+        ``pycode`` is imported in the following sequence:
+
+        - a user-provided path from CLI
+        - ``~/.andes/pycode``
+        - ``<andes_root>/pycode``
+
         """
-        pycode = None
+
         loaded = False
 
-        pycode = load_pycode_dot_andes()
+        # below are executed serially because of priority
+        pycode = reload_submodules('pycode')
         if not pycode:
-            # or use `pycode` in the andes source folder
-            try:
-                pycode = importlib.import_module("andes.pycode")
-                logger.info("Loaded generated Python code in andes source dir.")
-            except ImportError:
-                pass
+            pycode_path = get_pycode_path(self.options.get("pycode_path"), mkdir=False)
+            pycode = load_pycode_from_path(pycode_path)
+        if not pycode:
+            pycode = reload_submodules('andes.pycode')
+        if not pycode:
+            pycode = load_pycode_from_path(os.path.join(andes_root(), 'pycode'))
 
         # DO NOT USE `elif` here since below depends on the above.
         if pycode:
-            self._expand_pycode(pycode)
-            loaded = True
+            try:
+                self._expand_pycode(pycode)
+                loaded = True
+            except KeyError:
+                logger.error("Your generated pycode is broken. Run `andes prep` to re-generate. ")
 
         return loaded
 
@@ -1501,23 +1587,30 @@ class System:
 
         The output is an OrderedDict of model names and instances.
         """
-        if models is None:
-            models = self.exist.pflow
-        if isinstance(models, str):
-            models = {models: self.__dict__[models]}
+        out = OrderedDict()
+
+        if isinstance(models, OrderedDict):
+            out.update(models)
+
+        elif models is None:
+            out.update(self.exist.pflow)
+
+        elif isinstance(models, str):
+            out[models] = self.__dict__[models]
+
         elif isinstance(models, Model):
-            models = {models.class_name: models}
+            out[models.class_name] = models
+
         elif isinstance(models, list):
-            models = OrderedDict()
             for item in models:
                 if isinstance(item, Model):
-                    models[item.class_name] = item
+                    out[item.class_name] = item
                 elif isinstance(item, str):
-                    models[item] = self.__dict__[item]
+                    out[item] = self.__dict__[item]
                 else:
                     raise TypeError(f'Unknown type {type(item)}')
-        # do nothing for OrderedDict type
-        return models
+
+        return out
 
     def _store_tf(self, models):
         """
@@ -1583,34 +1676,43 @@ class System:
         """
         models_and_groups = list(self.models.values()) + list(self.groups.values())
 
+        # create an empty list of lists for all `BackRef` instances
         for model in models_and_groups:
             for ref in model.services_ref.values():
                 ref.v = [list() for _ in range(model.n)]
 
+        # `model` is the model who stores `IdxParam`s to other models
+        # `BackRef` is declared at other models specified by the `model` parameter
+        # of `IdxParam`s.
+
         for model in models_and_groups:
             if model.n == 0:
                 continue
+
+            # skip: a group is not allowed to link to other groups
             if not hasattr(model, "idx_params"):
-                # skip: group does not link to another group
                 continue
 
-            for ref in model.idx_params.values():
-                if ref.model not in self.models and (ref.model not in self.groups):
+            for idxp in model.idx_params.values():
+                if (idxp.model not in self.models) and (idxp.model not in self.groups):
                     continue
-                dest_model = self.__dict__[ref.model]
+                dest = self.__dict__[idxp.model]
 
-                if dest_model.n == 0:
+                if dest.n == 0:
                     continue
 
-                for n in (model.class_name, model.group):
-                    if n not in dest_model.services_ref:
+                for name in (model.class_name, model.group):
+                    # `BackRef` not requested by the linked models or groups
+                    if name not in dest.services_ref:
                         continue
 
-                    for model_idx, dest_idx in zip(model.idx.v, ref.v):
-                        if dest_idx not in dest_model.uid:
+                    for model_idx, dest_idx in zip(model.idx.v, idxp.v):
+                        if dest_idx not in dest.uid:
                             continue
-                        uid = dest_model.idx2uid(dest_idx)
-                        dest_model.services_ref[n].v[uid].append(model_idx)
+
+                        dest.set_backref(name,
+                                         from_idx=model_idx,
+                                         to_idx=dest_idx)
 
             # set model ``in_use`` flag
             if isinstance(model, Model):
@@ -1752,6 +1854,9 @@ class System:
         for instance in models.values():
             instance.switch_action(self.dae.t)
 
+        # TODO: generalize below for any models with timeseries data.
+        self.TimeSeries.apply_exact(self.dae.t)
+
     def _p_restore(self):
         """
         Restore parameters stored in `pin`.
@@ -1929,29 +2034,64 @@ class System:
 
         return out
 
+    def fix_address(self):
+        """
+        Fixes addressing issues after loading a snapshot.
 
-def load_pycode_dot_andes():
+        This function properly sets ``v`` and ``e`` of internal
+        variables as views of the corresponding DAE arrays.
+
+        Inputs will be refreshed for each model.
+        """
+
+        self.set_var_arrays(self.models)
+
+        for model in self.models.values():
+            model.get_inputs(refresh=True)
+
+        return True
+
+
+def load_pycode_from_path(pycode_path):
     """
     Helper function to load pycode from ``.andes``.
     """
-    pycode = None
 
-    MODULE_PATH = get_dot_andes_path() + '/pycode/__init__.py'
+    MODULE_PATH = os.path.join(pycode_path, '__init__.py')
     MODULE_NAME = 'pycode'
 
+    pycode = None
     if os.path.isfile(MODULE_PATH):
         try:
             spec = importlib.util.spec_from_file_location(MODULE_NAME, MODULE_PATH)
             pycode = importlib.util.module_from_spec(spec)  # NOQA
             sys.modules[spec.name] = pycode
             spec.loader.exec_module(pycode)
-            logger.info('Loaded generated Python code in "~/.andes/pycode".')
+            logger.info('Loaded generated Python code in "%s".', pycode_path)
         except ImportError:
-            pass
+            logger.debug('Failed loading generated Python code in "%s".', pycode_path)
 
     return pycode
 
 
+def reload_submodules(module_name):
+    """
+    Helper function for reloading an existing module and its submodules.
+
+    It is used to reload the ``pycode`` module after regenerating code.
+    """
+
+    if module_name in sys.modules:
+        pycode = sys.modules[module_name]
+        for _, m in inspect.getmembers(pycode, inspect.ismodule):
+            importlib.reload(m)
+
+        logger.info('Reloaded generated Python code of module "%s".', module_name)
+        return pycode
+
+    return None
+
+
 def _append_model_name(model_name, idx):
     """
     Helper function for appending ``idx`` to model names.
